from __future__ import absolute_import
import collections
from xml.parsers import expat


class ZcmlExtractor(object):
    ATTRIBUTES = set(['title', 'description'])

    def __call__(self, fileobj, keywords, comment_tags, options):
        self.keywords = keywords
        self.comment_tags = comment_tags
        self.options = options
        self.messages = []
        self.parser = expat.ParserCreate()
        self.parser.StartElementHandler = self.StartElementHandler
        self.parser.EndElementHandler = self.EndElementHandler
        self.domainstack = collections.deque()
        try:
            self.parser.ParseFile(fileobj)
        except expat.ExpatError:
            pass
        return self.messages

    def addMessage(self, message, comments=[]):
        self.messages.append(
                (self.parser.CurrentLineNumber, None, message, comments))

    def StartElementHandler(self, name, attributes):
        if 'i18n_domain' in attributes:
            self.domainstack.append(attributes["i18n_domain"])
        elif self.domainstack:
            self.domainstack.append(self.domainstack[-1])

        if not self.domainstack:
            return

        for (key, value) in attributes.items():
            if key in self.ATTRIBUTES:
                self.addMessage(value)

    def EndElementHandler(self, name):
        if self.domainstack:
            self.domainstack.pop()


def extract_zcml(fileobj, keywords, comment_tags, options):
    extractor = ZcmlExtractor()
    return extractor(fileobj, keywords, comment_tags, options)



from __future__ import absolute_import
import collections
import re
from StringIO import StringIO
from xml.parsers import expat

from lingua.extractors.python import PythonExtractor


class TranslateContext(object):
    WHITESPACE = re.compile(u"\s{2,}")
    EXPRESSION = re.compile(u"\s*\${[^}]*}\s*")

    def __init__(self, msgid, lineno, i18n_prefix):
        self.msgid = msgid
        self.text = []
        self.lineno = lineno
        self.i18n_prefix = i18n_prefix

    def addText(self, text):
        self.text.append(text)

    def addNode(self, name, attributes):
        name = attributes.get('%s:name' % self.i18n_prefix)
        if name:
            self.text.append(u'${%s}' % name)
        else:
            self.text.append(u'<dynamic element>')

    def ignore(self):
        text = u''.join(self.text).strip()
        text = self.WHITESPACE.sub(u' ', text)
        text = self.EXPRESSION.sub(u'', text)
        return not text

    def message(self):
        text = u''.join(self.text).strip()
        text = self.WHITESPACE.sub(u' ', text)
        if self.msgid:
            return (self.lineno, None, self.msgid, [u'Default: %s' % text])
        else:
            return (self.lineno, None, text, [])


class XmlExtractor(object):
    ENTITY = re.compile(r"&([A-Za-z]+|#[0-9]+);")
    UNDERSCORE_CALL = re.compile("_\(")

    def __call__(self, fileobj, keywords, comment_tags, options):
        self.keywords = keywords
        self.comment_tags = comment_tags
        self.options = options
        self.messages = []
        self.parser = expat.ParserCreate()
        self.parser.returns_unicode = True
        self.parser.UseForeignDTD()
        self.parser.SetParamEntityParsing(
            expat.XML_PARAM_ENTITY_PARSING_ALWAYS)
        self.parser.StartElementHandler = self.StartElementHandler
        self.parser.CharacterDataHandler = self.CharacterDataHandler
        self.parser.EndElementHandler = self.EndElementHandler
        self.parser.DefaultHandler = self.DefaultHandler
        self.domainstack = collections.deque()
        self.translatestack = collections.deque([None])
        self.prefix_stack = collections.deque(['i18n'])

        try:
            self.parser.ParseFile(fileobj)
        except expat.ExpatError:
            pass
        return self.messages

    def addMessage(self, message, comments=[]):
        self.messages.append(
                (self.parser.CurrentLineNumber, None, message, comments))

    def addUndercoreCalls(self, message):
        msg = message
        if isinstance(msg, unicode):
            msg = msg.encode('utf-8')
        py_extractor = PythonExtractor()
        py_messages = py_extractor(StringIO(msg), ['_'], None, None)
        for (line, _, py_message, comments) in py_messages:
            self.addMessage(py_message, comments)

    def StartElementHandler(self, name, attributes):
        i18n_prefix = self.prefix_stack[-1]
        for (attr, value) in attributes.items():
            if value == 'http://xml.zope.org/namespaces/i18n' and \
                    attr.startswith('xmlns:'):
                i18n_prefix = attr[6:]
        self.prefix_stack.append(i18n_prefix)

        new_domain = attributes.get('%s:domain' % i18n_prefix)
        if i18n_prefix and new_domain:
            self.domainstack.append(new_domain)
        elif self.domainstack:
            self.domainstack.append(self.domainstack[-1])

        if self.translatestack[-1]:
            self.translatestack[-1].addNode(name, attributes)

        i18n_translate = attributes.get('%s:translate' % i18n_prefix)
        if i18n_prefix and i18n_translate is not None:
            self.translatestack.append(TranslateContext(
                i18n_translate, self.parser.CurrentLineNumber, i18n_prefix))
        else:
            self.translatestack.append(None)

        if not self.domainstack:
            return

        i18n_attributes = attributes.get('%s:attributes' % i18n_prefix)
        if i18n_prefix and i18n_attributes:
            parts = [p.strip() for p in i18n_attributes.split(';')]
            for msgid in parts:
                if ' ' not in msgid:
                    if msgid not in attributes:
                        continue
                    self.addMessage(attributes[msgid])
                else:
                    try:
                        (msgid, attr) = msgid.split()
                    except ValueError:
                        continue
                    if attr not in attributes:
                        continue
                    self.addMessage(msgid, [u'Default: %s' % attributes[attr]])

        for (attr, value) in attributes.items():
            if self.UNDERSCORE_CALL.search(value):
                self.addUndercoreCalls(value)

    def DefaultHandler(self, data):
        if data.startswith(u'&') and self.translatestack[-1]:
            self.translatestack[-1].addText(data)

    def CharacterDataHandler(self, data):
        if TranslateContext.EXPRESSION.search(data) and \
                self.UNDERSCORE_CALL.search(data):
            self.addUndercoreCalls(data)
        if not self.translatestack[-1]:
            return

        data_length = len(data)
        context = self.parser.GetInputContext()

        while data:
            m = self.ENTITY.search(context)
            if m is None or m.start() >= data_length:
                self.translatestack[-1].addText(data)
                break

            n = self.ENTITY.match(data)
            if n is not None:
                length = n.end()
            else:
                length = 1

            self.translatestack[-1].addText(context[0: m.end()])
            data = data[m.start() + length:]

    def EndElementHandler(self, name):
        if self.prefix_stack:
            self.prefix_stack.pop()
        if self.domainstack:
            self.domainstack.pop()
        translate = self.translatestack.pop()
        if translate and not translate.ignore():
            self.messages.append(translate.message())


def extract_xml(fileobj, keywords, comment_tags, options):
    extractor = XmlExtractor()
    return extractor(fileobj, keywords, comment_tags, options)



import argparse
import os.path
import re
import sys
import xlrd
import xlwt
import polib


VARIABLE_RE = re.compile(r"\${(.*?)}")


def replace_catalog(filename, catalog):
    tmpfile = filename + '~'
    catalog.save(tmpfile)
    os.rename(tmpfile, filename)


def to_base26(value):
    if not value:
        return 'A'
    output = []
    while value:
        digit = value % 26
        if output:
            digit -= 1
        output.append(chr(ord('A') + digit))
        value /= 26
    return ''.join(reversed(output))


def cell_id(row, column):
    return '%s%d' % (to_base26(column), row + 1)


def cell_string(sheet, row, col):
    cell = sheet.cell(row, col)
    if cell.ctype != xlrd.XL_CELL_TEXT:
        return None
    return cell.value.strip()


def find_msg(sheet, row, catalog):
    msgid = cell_string(sheet, row, 0)
    if not msgid:
        print >> sys.stderr, ('Missing message id in cell %s:%s' %
                (sheet.name, cell_id(row, 0)))
        return None

    msg = catalog.find(msgid)
    if msg is None:
        print >> sys.stderr, \
                ('Can not find translation for cell %s:%s in PO file.' %
                (sheet.name, cell_id(row, 0)))
        return None

    return msg


def getVariables(text):
    variables = set(VARIABLE_RE.findall(text))
    if 'newline' in variables:
        variables.remove('newline')
    return variables


def update_message(msg,  sheet, row, column):
    if msg.comment.startswith(u'Default:'):
        canonical = msg.comment[9:]
    else:
        canonical = msg.msgid

    translation = cell_string(sheet, row, column)
    if not translation:
        return False

    fuzzy = False
    if getVariables(canonical) != getVariables(translation):
        print >> sys.stderr, \
                ('Bad translation in cell %s:%s: different variables used.' %
                (sheet.name, cell_id(row, column)))
        fuzzy = True

    msg.msgstr = translation
    if fuzzy and 'fuzzy' not in msg.flags:
        msg.flags.append('fuzzy')
    elif not fuzzy and 'fuzzy' in msg.flags:
        msg.flags.remove('fuzzy')


def ConvertXlsPo():
    parser = argparse.ArgumentParser(
            description='Merge translation from XLS file to a .PO file')

    parser.add_argument('locale', metavar='<locale>',
            help='Locale to process')
    parser.add_argument('input_file', metavar='<xls file>',
            help='Input XLS file')
    parser.add_argument('output_file', metavar='<po file>',
            help='PO file to update')
    options = parser.parse_args()

    book = xlrd.open_workbook(filename=options.input_file, logfile=sys.stderr)
    catalog = polib.pofile(options.output_file)

    found_locale = False
    for sheet in book.sheets():
        for col in range(2, sheet.ncols):
            context = cell_string(sheet, 0, col)
            if context != u'Translator comment' and context != options.locale:
                continue

            for row in range(1, sheet.nrows):
                msg = find_msg(sheet, row, catalog)
                if msg is None:
                    continue

                if context == u'Translator comment':
                    msg.tcomment = cell_string(sheet, row, col) or u''
                else:
                    found_locale = True
                    update_message(msg, sheet, row, col)

    if not found_locale:
        print >> sys.stderr, 'No translations found for locale %s' % \
                options.locale
        sys.exit(1)

    replace_catalog(options.output_file, catalog)


class CommentAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string):
        options = ['translator', 'extracted', 'reference']
        values = [v.strip() for v in values.split(',')]
        comments = set()
        for value in values:
            if value == 'all':
                comments.update(options)
                break
            elif value in options:
                comments.add(value)
            else:
                raise argparse.ArgumentError(self, 'Invalid comment option')
        setattr(namespace, self.dest, comments)


def ConvertPoXls():
    parser = argparse.ArgumentParser(
            description='Convert Po files for a domain to an Excel file.')
    parser.add_argument('--comments',
            action=CommentAction, default='',
            help='Comments to include in xls file. This is a comma separated '
                 'list of "translator", "extracted", "reference" or "all".')
    parser.add_argument('-p',
            dest='input', action='append', nargs=2, required=True,
            metavar=('<locale>', '<po file>'),
            help='Locale and filename of po-file to process')
    parser.add_argument('output_file', metavar='<xls file>',
            help='Output XLS file')
    options = parser.parse_args()

    catalogs = []
    for (locale, path) in options.input:
        catalogs.append((locale, polib.pofile(path)))

    messages = []
    seen = set()
    for catalog in catalogs:
        for msg in catalog[1]:
            if not msg.msgid:
                continue
            if msg.msgid not in seen:
                messages.append((msg.msgid, msg))
                seen.add(msg.msgid)

    book = xlwt.Workbook(encoding='utf-8')
    italic_style = xlwt.XFStyle()
    italic_style.num_format_str = 'Italic'
    italic_style.font.italic = True
    italic_style.font.bold = True
    sheet = book.add_sheet(u'Translations')
    column = 0
    sheet.write(0, column, u'Message id')
    column += 1
    if 'reference' in options.comments:
        sheet.write(0, column, u'References')
        column += 1
    if 'extracted' in options.comments:
        sheet.write(0, column, u'Source comment')
        column += 1
    if 'translator' in options.comments:
        sheet.write(0, column, u'Translator comment')
        column += 1
    for (i, cat) in enumerate(catalogs):
        sheet.write(0, column, cat[0])
        column += 1

    row = 1
    ref_catalog = catalogs[0][1]
    for (msgid, message) in messages:
        column = 0
        sheet.write(row, 0, msgid)
        column += 1
        msg = ref_catalog.find(msgid)
        if 'reference' in options.comments:
            o = []
            if msg is not None:
                for (entry, lineno) in msg.occurrences:
                    if lineno:
                        o.append(u'%s:%s' % (entry, lineno))
                    else:
                        o.append(entry)
            sheet.write(row, column, u', '.join(o))
            column += 1
        if 'extracted' in options.comments:
            if msg is not None:
                sheet.write(row, column, msg.comment)
            column += 1
        if 'translator' in options.comments:
            if msg is not None:
                sheet.write(row, column, msg.tcomment)
            column += 1
        for (i, cat) in enumerate(catalogs):
            cat = cat[1]
            msg = cat.find(msgid)
            if msg is not None:
                if 'fuzzy' in msg.flags:
                    sheet.write(row, column, msg.msgstr, italic_style)
                else:
                    sheet.write(row, column, msg.msgstr)
            column += 1
        row += 1

    book.save(options.output_file)



# coding=utf-8

import unittest


class ExtractTests(unittest.TestCase):
    def extract(self, snippet):
        from lingua.extractors.python import extract_python
        from StringIO import StringIO
        snippet = StringIO(snippet)
        return list(extract_python(snippet, ['_'], None, None))

    def test_multiline_string(self):
        self.assertEqual(
                self.extract("_('one two '\n'three')"),
                [(1, None, u'one two three', [])])



import argparse
import collections
import textwrap
import polib


def verify_po(path, show_path):
    leader = '[%s] ' % path if show_path else ''
    try:
        catalog = polib.pofile(path)
    except UnicodeDecodeError:
        print 'Character encoding problems occured while parsing %s' % path
        print 'Perhaps this is not a PO file?'
        return
    msgids = collections.defaultdict(int)
    reverse_map = collections.defaultdict(list)

    for entry in catalog:
        key = (entry.msgctxt, entry.msgid)
        msgids[key] += 1
        if entry.msgstr:
            reverse_map[entry.msgstr].append(key)

    for (key, count) in msgids.items():
        if count == 1:
            continue
        print '%sMessage repeated %d times:' % (leader, count)
        (context, msgid) = key
        if context:
            msgid = u'[%s] %s' % (context, msgid)
        print textwrap.fill(msgid, initial_indent=u' ' * 5,
                subsequent_indent=u' ' * 8).encode('utf-8')
        print

    for (msgstr, keys) in reverse_map.items():
        if len(keys) == 1:
            continue

        print '%sTranslation:' % leader
        print textwrap.fill(msgstr, initial_indent=u' ' * 8,
                subsequent_indent=u' ' * 8).encode('utf-8')
        print "Used for %d canonical texts:" % len(keys)
        for (idx, info) in enumerate(keys):
            (context, msgid) = info
            if context:
                msgid = u'[%s] %s' % (context, msgid)
            print textwrap.fill(msgid, initial_indent='%-8d' % (idx + 1),
                    subsequent_indent=8 * ' ').encode('utf-8')
        print


def main():
    parser = argparse.ArgumentParser(
            description="Perform sanity checks on PO files")
    parser.add_argument('input', metavar='PO-file', nargs='+',
            help='PO file to check')
    options = parser.parse_args()

    show_path = len(options.input) > 1
    for path in options.input:
        verify_po(path, show_path)



import unittest


class ExtractTests(unittest.TestCase):
    def extract(self, snippet):
        from lingua.extractors.zcml import extract_zcml
        from StringIO import StringIO
        snippet = StringIO(snippet)
        return list(extract_zcml(snippet, None, None, None))

    def testInvalidXML(self):
        self.assertEqual(self.extract(""), [])

    def testEmptyXml(self):
        self.assertEqual(self.extract("<configure/>"), [])

    def testi18nWithoutDomain(self):
        snippet = """\
                <configure>
                  <dummy title="test title"/>
                </configure>
                """
        self.assertEqual(self.extract(snippet), [])

    def testi18nWithDomain(self):
        snippet = """\
                <configure i18n_domain="lingua">
                  <dummy title="test title"/>
                </configure>
                """
        self.assertEqual(self.extract(snippet),
                [(2, None, "test title", [])])

    def testMultipleMessages(self):
        snippet = """\
                <configure i18n_domain="lingua">
                  <dummy title="test title 1"/>
                  <dummy title="test title 2"/>
                </configure>
                """
        self.assertEqual(self.extract(snippet),
                [(2, None, "test title 1", []),
                 (3, None, "test title 2", [])])

    def testDomainNesting(self):
        snippet = """\
                <configure>
                  <configure i18n_domain="lingua">
                      <dummy title="test title 1"/>
                  </configure>
                  <dummy title="test title 2"/>
                </configure>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, "test title 1", [])])



# coding=utf-8

import unittest


class ExtractTests(unittest.TestCase):
    def extract(self, snippet):
        from lingua.extractors.xml import extract_xml
        from StringIO import StringIO
        snippet = StringIO(snippet)
        return list(extract_xml(snippet, None, None, None))

    def testInvalidXML(self):
        self.assertEqual(self.extract(""), [])

    def testEmptyXml(self):
        self.assertEqual(self.extract("<html/>"), [])

    def test_attributes_plain(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy i18n:attributes="title" title="tést title"/>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"tést title", [])])

    def test_attributes_without_ns(self):
        snippet = """\
                <html>
                  <dummy i18n:translate="">Foo</dummy>
                  <other xmlns:i="http://xml.zope.org/namespaces/i18n"
                         i18n:domain="lingua">
                    <dummy i:translate="">Foo</dummy>
                  </other>
                  <dummy i18n:translate="">Foo</dummy>
                  <dummy i18n:translate="">Foo</dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [
                    (2, None, u"Foo", []),
                    (5, None, u"Foo", []),
                    (7, None, u"Foo", []),
                    (8, None, u"Foo", []),
                 ])

    def test_attributes_explicitMessageId(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy i18n:attributes="msg_title title" title="test tïtle"/>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"msg_title", [u"Default: test tïtle"])])

    def test_attributes_NoDomain(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n">
                  <dummy i18n:attributes="title" title="test title"/>
                </html>
                """
        self.assertEqual(self.extract(snippet), [])

    def test_attributes_multipleAttributes(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy i18n:attributes="title ; alt" title="tést title"
                         alt="test ålt"/>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"tést title", []),
                 (3, None, u"test ålt", [])])

    def test_attributes_multipleAttributesWithExplicitMessageId(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy i18n:attributes="msg_title title; msg_alt alt"
                         title="test titlé" alt="test ålt"/>
                </html>
                """
        self.assertEqual(sorted(self.extract(snippet)),
                [(3, None, "msg_alt", [u"Default: test ålt"]),
                 (3, None, "msg_title", [u"Default: test titlé"])])

    def test_translate_minimal(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy i18n:translate="">Dummy téxt</dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"Dummy téxt", [])])

    def test_translate_explicitMessageId(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy i18n:translate="msgid_dummy">Dummy téxt</dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"msgid_dummy", [u"Default: Dummy téxt"])])

    def test_translate_subelement(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy i18n:translate="msgid_dummy">Dummy
                    <strong>text</strong> demø</dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, "msgid_dummy",
                    [u"Default: Dummy <dynamic element> demø"])])

    def test_translate_namedSubelement(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy i18n:translate="msgid_dummy">Dummy
                    <strong i18n:name="text">téxt</strong> demø</dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"msgid_dummy", [u"Default: Dummy ${text} demø"])])

    def test_translate_translatedSubElement(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy i18n:translate="msgid_dummy">Dummy
                    <strong i18n:name="text"
                            i18n:translate="msgid_text">téxt</strong>
                    demø</dummy>
                </html>
                """
        self.assertEqual(sorted(self.extract(snippet)),
                [(3, None, u"msgid_dummy", [u"Default: Dummy ${text} demø"]),
                 (4, None, u"msgid_text", [u"Default: téxt"])])

    def test_translate_stripExtraWhitespace(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy i18n:translate="">Dummy


                  text</dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"Dummy text", [])])

    def test_translate_stripTrailingAndLeadingWhitespace(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy i18n:translate="">
                      Dummy text
                  </dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"Dummy text", [])])

    def test_translate_HtmlEntity(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <button i18n:translate="">lock &amp; load&nbsp;</button>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"lock &amp; load&nbsp;", [])])

    def test_ignore_undeclared_namespace(self):
        snippet = """\
                <html xmlns="http://www.w3.org/1999/xhtml"
                      xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <tal:block/>
                  <p i18n:translate="">Test</p>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(5, None, u"Test", [])])

    def test_ignore_dynamic_message(self):
        snippet = """\
                <html xmlns="http://www.w3.org/1999/xhtml"
                      xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <p i18n:translate="">${'dummy'}</p>
                </html>
                """
        self.assertEqual(self.extract(snippet), [])

    def test_translate_underscore_call_with_unicode_arg(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy tal:replace="_(u'foo')">Dummy</dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"foo", [])])

    def test_translate_underscore_call_with_str_arg(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy tal:replace='_("foo")'>Dummy</dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"foo", [])])

    def test_translate_underscore_call_with_parenthesis(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy tal:replace='_("f (o) o")'>Dummy</dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"f (o) o", [])])

    def test_translate_underscore_call_with_default(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy tal:replace='_("foo", default="blah")'>Dummy</dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"foo", [u'Default: blah'])])

    def test_translate_multiple_underscore_calls(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy \
                tal:replace="multiple(_('foo'), _('blah'))">Dummy</dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"foo", []),
                 (3, None, u"blah", [])])

    def test_translate_inner_underscore_call(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy>${_("foo")}</dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"foo", [])])

    def test_translate_inner_multiple_underscore_calls(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy>${dummy(_("foo"), _("bar"))}</dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"foo", []),
                 (3, None, u"bar", [])])

    def test_translate_inner_underscore_call_with_default(self):
        snippet = """\
                <html xmlns:i18n="http://xml.zope.org/namespaces/i18n"
                      i18n:domain="lingua">
                  <dummy>${_("foo", default="blah")}>Dummy</dummy>
                </html>
                """
        self.assertEqual(self.extract(snippet),
                [(3, None, u"foo", [u'Default: blah'])])



import tokenize
from babel.util import parse_encoding


def safe_eval(s, encoding='ascii'):
    if encoding != 'ascii':
        s = s.decode(encoding)
    return eval(s, {'__builtins__': {}}, {})


class PythonExtractor(object):
    def __call__(self, fileobj, keywords, comment_tags, options):
        self.state = self.stateWaiting
        self.msg = None
        self.keywords = keywords
        self.messages = []
        self.encoding = parse_encoding(fileobj) or "ascii"
        tokens = tokenize.generate_tokens(fileobj.readline)
        for (ttype, tstring, stup, etup, line) in tokens:
            self.state(ttype, tstring, stup[0])
        return self.messages

    def stateWaiting(self, ttype, tstring, lineno):
        if ttype == tokenize.NAME and tstring in self.keywords:
            self.state = self.stateKeywordSeen
            self.msg = dict(lineno=lineno)

    def stateKeywordSeen(self, ttype, tstring, lineno):
        # We have seen _, now check if this is a _( .. ) call
        if ttype == tokenize.OP and tstring == '(':
            self.state = self.stateWaitForLabel
        else:
            self.state = self.stateWaiting

    def stateWaitForLabel(self, ttype, tstring, lineno):
        # We saw _(, wait for the message label
        if ttype == tokenize.STRING:
            self.msg.setdefault('label', []).append(
                    safe_eval(tstring, self.encoding))
        elif ttype == tokenize.OP and tstring == ',':
            self.state = self.stateWaitForDefault
        elif ttype == tokenize.OP and tstring == ')':
            self.addMessage(self.msg)
            self.state = self.stateWaiting
        elif ttype == tokenize.NAME:
            self._parameter = tstring
            self.state = self.stateInFactoryParameter
        elif ttype == tokenize.NL:
            pass
        else:
            # Effectively a syntax error, but ignore and reset state
            self.msg = None
            self.state = self.stateWaiting

    def stateWaitForDefault(self, ttype, tstring, lineno):
        # We saw _('label', now wait for a default translation
        if ttype == tokenize.STRING:
            self.msg.setdefault('default', []).append(
                    safe_eval(tstring, self.encoding))
        elif ttype == tokenize.NAME:
            self._parameter = tstring
            self.state = self.stateInFactoryParameter
        elif ttype == tokenize.OP and tstring == ',':
            self.state = self.stateInFactoryWaitForParameter
        elif ttype == tokenize.OP and tstring == ')':
            self.addMessage(self.msg)
            self.state = self.stateWaiting
        # We ignore anything else (ie whitespace, comments, syntax errors,
        # etc.)

    def stateInFactoryWaitForParameter(self, ttype, tstring, lineno):
        if ttype == tokenize.OP and tstring == ')':
            self.addMessage(self.msg)
            self.msg = None
            self.state = self.stateWaiting
        elif ttype == tokenize.NAME:
            self._parameter = tstring
            self.state = self.stateInFactoryParameter

    def stateInFactoryParameter(self, ttype, tstring, lineno):
        if ttype == tokenize.STRING:
            self.msg.setdefault(self._parameter, []).append(
                    safe_eval(tstring, self.encoding))
        elif ttype == tokenize.OP and tstring == ',':
            self.state = self.stateInFactoryWaitForParameter
        elif ttype == tokenize.OP and tstring == ')':
            self.addMessage(self.msg)
            self.state = self.stateWaiting

    def addMessage(self, msg):
        if not msg.get('label'):
            return
        default = msg.get('default', None)
        if default:
            comments = [u'Default: %s' % u''.join(default)]
        else:
            comments = []
        self.messages.append(
                (msg['lineno'], None, u''.join(msg['label']), comments))


extract_python = PythonExtractor()



from setuptools import setup, find_packages
import sys

version = "1.2"

install_requires=[
        "polib",
        "xlrd",
        "xlwt",
        ]
if sys.version_info<(2,7):
    install_requires.append("argparse")

setup(name="lingua",
      version=version,
      description="Translation toolset",
      long_description=open('README.rst').read() + '\n' + \
              open('changes.rst').read(),
      classifiers=[
          "Intended Audience :: Developers",
          "License :: DFSG approved",
          "License :: OSI Approved :: BSD License",
          "Operating System :: OS Independent",
          "Programming Language :: Python :: 2",
          "Programming Language :: Python :: 2.6",
          "Programming Language :: Python :: 2.7",
          "Topic :: Software Development :: Libraries :: Python Modules",
          ],
      keywords='translation po gettext Babel',
      author="Wichert Akkerman",
      author_email="wichert@wiggy.net",
      url='https://github.com/wichert/lingua',
      license='BSD',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      include_package_data=True,
      zip_safe=True,
      install_requires=install_requires,
      entry_points="""
      [console_scripts]
      po-to-xls = lingua.xlsconvert:ConvertPoXls
      xls-to-po = lingua.xlsconvert:ConvertXlsPo
      polint = lingua.polint:main

      [babel.extractors]
      lingua_python = lingua.extractors.python:extract_python
      lingua_xml = lingua.extractors.xml:extract_xml
      lingua_zcml = lingua.extractors.zcml:extract_zcml
      """
      )
